@@ -13,11 +13,12 @@ import java.util.Vector;
 
 import org.gnu.gdk.Point;
 import org.gnu.glib.EventMap;
+import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
+import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.EntryEvent;
 import org.gnu.gtk.event.EntryListener;
-import org.gnu.gtk.event.GtkEventType;
 
 /**
  * A single line text entry widget. A fairly large set of key bindings are 
@@ -34,7 +35,7 @@ public class Entry extends Widget implements Editable {
 		super(gtk_entry_new ());
 	}
 	
-	public Entry(int handle) {
+	public Entry(Handle handle) {
 	    super(handle);
 	}
 
@@ -44,6 +45,8 @@ public class Entry extends Widget implements Editable {
 	 * @param text The new text.
 	 */
 	public void setText(String text){
+	    if (null == text)
+	        text = "";
 		gtk_entry_set_text(getHandle(), text);
 	}
 
@@ -74,12 +77,13 @@ public class Entry extends Widget implements Editable {
 	}
 
 	/**
-	 * Sets the character to use in place of the actual text when setVisibility
-	 * has been called to set text visibility to FALSE. i.e. this is the character 
-	 * used in "password mode" to show the user how many characters have been 
-	 * typed. The default invisible char is an asterisk ('*'). If you set the 
-	 * invisible char to 0, then the user will get no feedback at all; there will 
-	 * be no text on the screen as they type.
+	 * Sets the character to use in place of the actual text when
+	 * setVisibility has been called to set text visibility to
+	 * FALSE (ie this is the character used in "password mode" to
+	 * show the user how many characters have been typed). The
+	 * default invisible char is an asterisk ('*'). If you set the
+	 * invisible char to 0, then the user will get no feedback at
+	 * all; there will be no text on the screen as they type.
 	 */
 	public void setInvisibleChar(char character){
 		gtk_entry_set_invisible_char(getHandle(), (byte) character);
@@ -157,8 +161,8 @@ public class Entry extends Widget implements Editable {
 	 * @return The PangoLayout for this entry
 	 */
 	public org.gnu.pango.Layout getLayout(){
-		int hndl = gtk_entry_get_layout(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_entry_get_layout(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (org.gnu.pango.Layout)obj;
 		return new org.gnu.pango.Layout(hndl);
@@ -179,6 +183,43 @@ public class Entry extends Widget implements Editable {
 		return new Point(x[0], y[0]);
 	}
 
+    /**
+     * Sets the alignment for the contents of the entry. This controls
+     * the horizontal positioning of the contents when the displayed
+     * text is shorter than the width of the entry.
+     *
+     * @param xalign The horizontal alignment, from 0 (left) to 1
+     * (right). Reversed for RTL layouts.
+     */
+    public void setAlignment( float xalign ) {
+        gtk_entry_set_alignment(getHandle(), xalign);
+    }
+    /**
+     * Gets the value set by {@link #setAlignment}.
+     */
+    public float getAlignment() {
+        return gtk_entry_get_alignment(getHandle());
+    }
+    /**
+     * Converts from a position in the entry contents (returned by
+     * {@link #getText}) to a position in the entry's {@link
+     * org.gnu.pango.Layout PangoLayout} (returned by {@link
+     * #getLayout}, with text retrieved via {@link
+     * org.gnu.pango.Layout#getText}).
+     */
+    public int layoutIndexToTextIndex( int layoutIndex ) {
+        return gtk_entry_layout_index_to_text_index(getHandle(), layoutIndex);
+    }
+    /**
+     * Converts from a position in the entry's {@link
+     * org.gnu.pango.Layout PangoLayout} (returned by {@link
+     * #getLayout}) to a position in the entry contents (returned by
+     * {@link #getText}).
+     */
+    public int textIndexToLayoutIndex( int textIndex ) {
+        return gtk_entry_text_index_to_layout_index(getHandle(), textIndex);
+    }
+
 	/**
 	 * Retrieve the runtime type used by the GLib library.
 	 */
@@ -256,8 +297,8 @@ public class Entry extends Widget implements Editable {
 		return cls;
 	}
 
-	public GtkEventType getEventType(String signal) {
-		GtkEventType et = evtMap.getEventType(signal);
+	public EventType getEventType(String signal) {
+		EventType et = evtMap.getEventType(signal);
 		if (et == null) et = super.getEventType(signal);
 		return et;
 	}
@@ -352,8 +393,8 @@ public class Entry extends Widget implements Editable {
 	}
 	
 	public EntryCompletion getCompletion() {
-		int hndl = gtk_entry_get_completion(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_entry_get_completion(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (EntryCompletion)obj;
 		return new EntryCompletion(hndl);
@@ -415,33 +456,38 @@ public class Entry extends Widget implements Editable {
 	}
 	
 
-	/****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
     native static final protected int gtk_entry_get_type ();
-    native static final protected int gtk_entry_new ();
-    native static final protected void gtk_entry_set_visibility (int entry, boolean visible);
-    native static final protected boolean gtk_entry_get_visibility (int entry);
-    native static final protected void gtk_entry_set_invisible_char (int entry, byte ch);
-    native static final protected byte gtk_entry_get_invisible_char (int entry);
-    native static final protected void gtk_entry_set_has_frame (int entry, boolean setting);
-    native static final protected boolean gtk_entry_get_has_frame (int entry);
-    native static final protected void gtk_entry_set_max_length (int entry, int max);
-    native static final protected int gtk_entry_get_max_length (int entry);
-    native static final protected void gtk_entry_set_activates_default (int entry, boolean setting);
-    native static final protected boolean gtk_entry_get_activates_default (int entry);
-    native static final protected void gtk_entry_set_width_chars (int entry, int numChars);
-    native static final protected int gtk_entry_get_width_chars (int entry);
-    native static final protected void gtk_entry_set_text (int entry, String text);
-    native static final protected String gtk_entry_get_text (int entry);
-    native static final protected int gtk_entry_get_layout (int entry);
-    native static final protected void gtk_entry_get_layout_offsets (int entry, int [] x, int [] y);
+    native static final protected Handle gtk_entry_new ();
+    native static final protected void gtk_entry_set_visibility (Handle entry, boolean visible);
+    native static final protected boolean gtk_entry_get_visibility (Handle entry);
+    native static final protected void gtk_entry_set_invisible_char (Handle entry, byte ch);
+    native static final protected byte gtk_entry_get_invisible_char (Handle entry);
+    native static final protected void gtk_entry_set_has_frame (Handle entry, boolean setting);
+    native static final protected boolean gtk_entry_get_has_frame (Handle entry);
+    native static final protected void gtk_entry_set_max_length (Handle entry, int max);
+    native static final protected int gtk_entry_get_max_length (Handle entry);
+    native static final protected void gtk_entry_set_activates_default (Handle entry, boolean setting);
+    native static final protected boolean gtk_entry_get_activates_default (Handle entry);
+    native static final protected void gtk_entry_set_width_chars (Handle entry, int numChars);
+    native static final protected int gtk_entry_get_width_chars (Handle entry);
+    native static final protected void gtk_entry_set_text (Handle entry, String text);
+    native static final protected String gtk_entry_get_text (Handle entry);
+    native static final protected Handle gtk_entry_get_layout (Handle entry);
+    native static final protected void gtk_entry_get_layout_offsets (Handle entry, int [] x, int [] y);
+    native static final protected void gtk_entry_set_completion(Handle entry, Handle completion);
+    native static final protected Handle gtk_entry_get_completion(Handle entry);
+    native static final private void gtk_entry_set_alignment(Handle entry, float xalign);
+    native static final private float gtk_entry_get_alignment(Handle entry);
+    native static final private int gtk_entry_layout_index_to_text_index(Handle entry, int layout_index);
+    native static final private int gtk_entry_text_index_to_layout_index(Handle entry, int text_index);
 
-    // new methods for 2.5.1
-    native static final protected void gtk_entry_set_completion(int entry, int completion);
-    native static final protected int gtk_entry_get_completion(int entry);
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
+    /* Deprecated functions.
+    native static final private Handle gtk_entry_new_with_max_length(int max);
+    native static final private void gtk_entry_append_text(Handle entry, String text);
+    native static final private void gtk_entry_prepend_text(Handle entry, String text);
+    native static final private void gtk_entry_set_position(Handle entry, int position);
+    native static final private void gtk_entry_select_region(Handle entry, int start, int end);
+    native static final private void gtk_entry_set_editable(Handle entry, boolean editable);
+    */
 }
 
